@@ -29,6 +29,12 @@ test_that("Tidy results of simulations from binomial-outcome trial without commo
   expect_snapshot(extract_results(res, raw_ests = TRUE))
 })
 
+test_that("Sequential and parallel extraction works similarly", {
+  res <- read_testdata("binom__results__3_arms__common_control__equivalence__futility__softened")
+  expect_identical(extract_results(res, cores = 1),
+                   extract_results(res, cores = 2))
+})
+
 test_that("Metric history of specific trial works", {
   res <- read_testdata("binom__result__3_arms__common_control__equivalence__futility__softened")
   expect_snapshot(extract_history(res))
@@ -56,4 +62,7 @@ test_that("Extract results erors correctly on invalid inputs", {
   expect_error(extract_results(res, select_last_arm = NA))
   expect_error(extract_results(res_no_cont, select_last_arm = TRUE))
   expect_error(extract_results(res_no_cont, te_comp = "Wrong arm"))
+
+  # Wrong number of cores
+  expect_error(extract_results(res, cores = 0.9))
 })
